import { parceable } from '../Schema'
import SchemaItem from '../SchemaItem'
import { SchemaInfer, ValidationError, ValidationResult } from '../types'

export default class SchemaArray<Type extends SchemaItem> extends SchemaItem<Array<SchemaInfer<Type>>> {
	public constructor(public readonly values: Type) { super([values]) }

	/**
	 * transform the array so it only contains one of each elements
	 */
	@parceable()
	public unique(): this {
		this.postProcess.push((input) => input.filter((it, idx) => input.indexOf(it) === idx))

		return this
	}

	/**
	* split a string into an array
	* @param [separator] default:`,` the separator to use
	*/
	public split(separator = ',') {
		this.addPreProcess((it) => typeof it === 'string' ? it.split(separator) : it)
	}

	public parse(input: unknown, options?: { fast?: boolean }): ValidationResult<Array<SchemaInfer<Type>>> {

		// check errors from itself
		const { valid, object, errors = [] } = super.parse(input, options)

		// skip checking childs if self is not valid (maybe still try to check childs whan fast is false ?)
		if (!valid) {
			return {
				valid,
				object,
				errors
			} as ValidationResult<SchemaInfer<Type>>
		}

		const clone: Array<SchemaInfer<Type>> = []
		const errs: Array<ValidationError> = []
		for (let idx = 0; idx < (object as Array<Type>).length; idx++) {
			const item = (object as Array<Type>)[idx]
			const res: ValidationResult<SchemaInfer<Type>> = this.values.parse(item)

			// handle valid child schema
			if (res.valid) {
				clone.push(res.object)
				continue
			}

			// handle child errors
			const errss = res.errors.map((it) => ({
				...it,
				field: it.field ? `${idx}.${it.field}` : idx.toString()
			}))

			if (options?.fast) {
				return {
					valid: false,
					object: clone,
					errors: errss
				}
			}
			errs.push(...errss)
		}

		if (errs.length > 0) {
			return {
				valid: false,
				object: clone,
				errors: errs
			}
		}

		return {
			valid: true,
			object: clone
		}
	}


	public override unwrap(): Type {
		return this.values
	}

	public override isOfType(input: unknown): input is Array<SchemaInfer<Type>> {
		return Array.isArray(input)
	}
}
